"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var prop_types_1 = require("prop-types");
var _ = require("underscore");
var formCore_1 = require("./formCore");
var formBinders_1 = require("./formBinders");
var properyPathResolver_1 = require("./properyPathResolver");
var classNames_1 = require("../../utilities/classNames");
/** The default Json Entity binder - NOTE, the original instance provided is MUTABLE */
var JsonEntityBinder = (function () {
    function JsonEntityBinder(data) {
        this.data = data;
    }
    JsonEntityBinder.prototype.getValue = function (dataPath) {
        return properyPathResolver_1.PropertyPathResolver.getValue(this.data, dataPath);
    };
    JsonEntityBinder.prototype.setValue = function (dataPath, value) {
        properyPathResolver_1.PropertyPathResolver.setValue(this.data, dataPath, value);
        this.lastDataPathSet = dataPath;
    };
    JsonEntityBinder.prototype.toJson = function () {
        return this.data;
    };
    return JsonEntityBinder;
}());
var FormDataClone = (function () {
    function FormDataClone() {
    }
    FormDataClone.json = function (source) {
        return JSON.parse(JSON.stringify(source));
    };
    FormDataClone.custom = function (source) {
        var clone = _.clone(source);
        _.keys(clone).map(function (key) {
            var value = clone[key];
            if (_.isObject(value)) {
                clone[key] = FormDataClone.custom(value);
            }
        });
        return clone;
    };
    return FormDataClone;
}());
exports.FormDataClone = FormDataClone;
function generateUniqueId(formatter) {
    var u = "" + Math.random();
    return formatter ? formatter(u) : u;
}
exports.generateUniqueId = generateUniqueId;
function extractChildValidationResults(validationResults, dataPath) {
    var vr = validationResults && _.filter(validationResults, function (vr) { return vr.attribute.indexOf(dataPath + ".") === 0; });
    if (!vr) {
        return;
    }
    return vr.map(function (a) { return (tslib_1.__assign({}, a, { attribute: a.attribute.substring(dataPath.length + 1) })); });
}
exports.extractChildValidationResults = extractChildValidationResults;
/**
A stateless data bindable form - state is held within the 'dataBinder' property
NOTE: This is designed to render all elements in the form on every change. This allows other participating elements to react to these changes
NOTE: This element provides a react context, this can be used to get access to the Forms dataBinder (or any parent Form dataBinder when nested)
*/
var Form = (function (_super) {
    tslib_1.__extends(Form, _super);
    function Form() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.preventDefault = function (e) { e.preventDefault(); return false; };
        _this.notifyChange = function () {
            _this.props.onDataBinderChange && _this.props.onDataBinderChange(_this.props.dataBinder);
            _this.props.onDataChanged && _this.props.onDataChanged(_this.props.dataBinder.toJson());
        };
        return _this;
    }
    Form.getFormContext = function (context) {
        return context["form"];
    };
    Form.prototype.getChildContext = function () {
        return {
            form: {
                dataBinder: this.props.dataBinder,
                parent: this.context["form"]
            }
        };
    };
    Form.jsonDataBinder = function (data) {
        return new JsonEntityBinder(data);
    };
    Form.jsonDataBinderWithClone = function (data) {
        return new JsonEntityBinder(FormDataClone.custom(data));
    };
    Form.prototype.render = function () {
        var ch = this.processChildren(this.props.children, this.props.validationMode);
        var hasParentForm = !!Form.getFormContext(this.context);
        var className = classNames_1.ClassHelpers.classNames("form", hasParentForm && "form-nested", this.props.className);
        return hasParentForm ? React.createFactory("div")({ className: className }, ch) : React.createFactory("form")({ className: className, onSubmit: this.preventDefault }, ch);
    };
    Form.prototype.processChildren = function (node, validationMode) {
        var _this = this;
        var validationResults = this.props.validationResults;
        return React.Children.map(node, function (element) {
            if (!element) {
                return;
            }
            if (!element.props) {
                return element;
            }
            var props = _.extend({}, element.props);
            if (validationMode && props["validationMode"]) {
                props["validationMode"] = validationMode;
            }
            var children = element.props.children;
            var fbi = props;
            var formBinder = formCore_1.getFormBinderFromInjector(fbi);
            if (formBinder) {
                formCore_1.updateFormBinderInjector(fbi, null);
                if (validationResults && validationResults.length) {
                    var vr = _.find(validationResults, function (vr) { return vr.attribute === formBinder.dataPath; });
                    if (vr) {
                        props["data-validation-message"] = vr.message;
                    }
                    else {
                        // use for child form
                        props["validationResults"] = extractChildValidationResults(validationResults, formBinder.dataPath);
                    }
                    if (validationMode && props["validationMode"]) {
                        props["validationMode"] = validationMode;
                    }
                }
                formBinder.setElementProperty(props, _this.props.dataBinder);
                formBinder.handleValueChanged(props, _this.props.dataBinder, _this.notifyChange);
                if (formBinder.extender) {
                    formBinder.extender(props, _this.props.dataBinder, _this.notifyChange);
                }
            }
            else if (children) {
                children = _this.processChildren(children, validationMode);
            }
            return React.cloneElement(element, props, children);
        });
    };
    Form.defaultProps = {
        validationMode: "icon"
    };
    Form.contextTypes = { "form": prop_types_1.PropTypes.object };
    Form.childContextTypes = {
        form: prop_types_1.PropTypes.object
    };
    Form.Bind = formBinders_1.FormBinder;
    return Form;
}(React.Component));
exports.Form = Form;
